part of '../crypto_revolution_masters_api.dart';

class User {
  final CryptoRevolutionMasterApp _cryptoRevolutionMasterApp;

  User({
    required CryptoRevolutionMasterApp cryptoRevolutionMasterApp
  }): 
    _cryptoRevolutionMasterApp = cryptoRevolutionMasterApp;

  Future<void> deleteFavouriteCoins({ required String id, required List<String> coins }) async {
    if (_cryptoRevolutionMasterApp.isInitialize) {
      var headersList = {
        'Accept': '*/*',
        'Content-Type': 'application/json' 
      };
      var url = Uri.parse('${_cryptoRevolutionMasterApp.appUrl}/user/$id/favourite-coins');

      var req = http.Request('DELETE', url);
      req.headers.addAll(headersList);
      req.body = json.encode(coins);

      var res = await req.send();
      final resBody = await res.stream.bytesToString();

      if (res.statusCode >= 200 && res.statusCode < 300) {
        Map<String, dynamic> resBodyJson = json.decode(resBody);
        if (resBodyJson.containsKey("reason")) {
          if (resBodyJson["reason"] == "user doesn't not exist") {
            return Future.error(ResponseErrorMessage.userDoseNotExist);
          } else {
            return Future.error(ResponseErrorMessage.backendError);
          }
        }
      }
      else {
        return Future.error(ResponseErrorMessage.backendError);
      }
    } else {
      return Future.error(ResponseErrorMessage.invalidedSecret);
    }
  }

  Future<void> addFavouriteCoins({required String id, required List<String> coins}) async {
    if (_cryptoRevolutionMasterApp.isInitialize) {
      var headersList = {
        'Accept': '*/*',
        'Content-Type': 'application/json' 
      };
      var url = Uri.parse('${_cryptoRevolutionMasterApp.appUrl}/user/$id/favourite-coins');

      var req = http.Request('POST', url);
      req.headers.addAll(headersList);
      req.body = json.encode(coins);

      var res = await req.send();
      final resBody = await res.stream.bytesToString();

      if (res.statusCode >= 200 && res.statusCode < 300) {
        Map<String, dynamic> resBodyJson = json.decode(resBody);
        if (resBodyJson.containsKey("reason")) {
          if (resBodyJson["reason"] == "user doesn't not exist") {
            return Future.error(ResponseErrorMessage.userDoseNotExist);
          } else {
            return Future.error(ResponseErrorMessage.backendError);
          }
        }
      }
      else {
        return Future.error(ResponseErrorMessage.backendError);
      }
    } else {
      return Future.error(ResponseErrorMessage.invalidedSecret);
    }
  }

  Future<List<CoinModel>> getFavouriteCoins({required String id}) async {
    if (_cryptoRevolutionMasterApp.isInitialize) {
      var headersList = {
        'Accept': '*/*',
      };
      var url = Uri.parse('${_cryptoRevolutionMasterApp.appUrl}/user/$id/favourite-coins');

      var req = http.Request('GET', url);

      req.headers.addAll(headersList);

      var res = await req.send();
      final resBody = await res.stream.bytesToString();

      if (res.statusCode >= 200 && res.statusCode < 300) {
        Map<String, dynamic> resBodyJson = json.decode(resBody);
        if (resBodyJson.containsKey("reason")) {
          if (resBodyJson["reason"] == "user doesn't not exist") {
            return Future.error(ResponseErrorMessage.userDoseNotExist);
          } else {
            return Future.error(ResponseErrorMessage.backendError);
          }
        } else {
          List<CoinModel> filteredData = List.empty(growable: true);
          for (var element in (resBodyJson["data"] as List<dynamic>)) { 
            filteredData.add(
              CoinModel(
                name: element["name"], 
                symbol: element["symbol"], 
                image: element["image"], 
                current_price: element["current_price"], 
                market_cap: element["market_cap"], 
                market_cap_rank: element["market_cap_rank"], 
                fully_diluted_valuation: element["fully_diluted_valuation"], 
                market_cap_change_24h: element["market_cap_change_24h"], 
                ath_change_percentage: element["ath_change_percentage"], 
                market_cap_change_percentage_24h: element["market_cap_change_percentage_24h"], 
                high_24h: element["high_24h"], 
                low_24h: element["low_24h"]
              )
            );
          }
          return Future.value(filteredData);
        }
      }
      else {
        return Future.error(ResponseErrorMessage.backendError);
      }
    } else {
      return Future.error(ResponseErrorMessage.invalidedSecret);
    }
  }

  Future<List<PortfolioCoinModel>> portfolio({required String id}) async {
    if (_cryptoRevolutionMasterApp.isInitialize) {
      int statusCode = 0;

      dynamic resBody;

      var headersList = {
        'Accept': '*/*',
      };
      var url = Uri.parse('${_cryptoRevolutionMasterApp.appUrl}/user/$id/portfolio');

      var req = http.Request('GET', url);
      req.headers.addAll(headersList);

      var res = await req.send();

      statusCode = res.statusCode;

      resBody = await res.stream.bytesToString();

      if (statusCode != 0) {
        if (res.statusCode >= 200 && res.statusCode < 300) {
          Map<String, dynamic> data = json.decode(resBody);
          if (data.containsKey("data")) {
            List<PortfolioCoinModel> filteredData = List.empty(growable: true);
              for (var element in (data["data"] as List<dynamic>)) { 
                filteredData.add(
                  PortfolioCoinModel(
                    name: element["name"], 
                    symbol: element["symbol"], 
                    image: element["image"], 
                    current_price: element["current_price"], 
                    market_cap: element["market_cap"], 
                    market_cap_rank: element["market_cap_rank"], 
                    fully_diluted_valuation: element["fully_diluted_valuation"], 
                    market_cap_change_24h: element["market_cap_change_24h"], 
                    ath_change_percentage: element["ath_change_percentage"], 
                    market_cap_change_percentage_24h: element["market_cap_change_percentage_24h"], 
                    high_24h: element["high_24h"], 
                    low_24h: element["low_24h"],
                    amount: element["amount"],
                    amount_in_current_price: element["amount_in_current_price"]
                  )
                );
              }
              return Future.value(filteredData);
          } else {
            if (data["reason"] == "user doesn't exist") {
              return Future.error(ResponseErrorMessage.userDoseNotExist);
            } else {
              return Future.error(ResponseErrorMessage.backendError);
            }
          }
        } else if (statusCode > 300 && statusCode <= 400) {
          return Future.error(ResponseErrorMessage.noInternetConnection);
        } else {
          return Future.error(ResponseErrorMessage.backendError);
        }
      } else {
        return Future.error(ResponseErrorMessage.backendError);
      }
    } else {
      return Future.error(ResponseErrorMessage.invalidedSecret);
    }
  }

  Future<UserModel> edit({
    required String id,
    String? email,
    String? password,
    String? username

  }) async {
    if (_cryptoRevolutionMasterApp.isInitialize) {
      var headersList = {
        'Accept': '*/*',
        'Content-Type': 'application/json' 
      };
      var url = Uri.parse('${_cryptoRevolutionMasterApp.appUrl}/user/$id');

      var body = {};
      if (email != null) {
        body["email"] = email;
      }
      if (password != null) {
        body["password"] = password;
      }
      if (username != null) {
        body["username"] = username;
      }
      var req = http.Request('PUT', url);
      req.headers.addAll(headersList);
      req.body = json.encode(body);
      var res = await req.send();
      final resBody = await res.stream.bytesToString();

      if (res.statusCode >= 200 && res.statusCode < 300) {
        Map<String, dynamic> resBodyJson = json.decode(resBody);
        if (resBodyJson.containsKey("data")) {
          return Future.value(
            UserModel(
              id: id, 
              email: resBodyJson["data"]["email"], 
              username: resBodyJson["data"]["username"], 
              password: resBodyJson["data"]["password"]
            )
          );
        } else {
          if (resBodyJson["reason"] == "user doesn't not exist") {
            return Future.error(ResponseErrorMessage.userDoseNotExist);
          } else if (resBodyJson["reason"] == "username and email already in use") {
            return Future.error(ResponseErrorMessage.usernameAndEmailAlreadyInUse);
          } else if (resBodyJson["reason"] == "email already in use") {
            return Future.error(ResponseErrorMessage.emailAlreadyInUse);
          } else if (resBodyJson["reason"] == "username already in use") {
            return Future.error(ResponseErrorMessage.usernameAlreadyInUse);
          } else {
            return Future.error(ResponseErrorMessage.backendError);
          }
        }
      }
      else {
        return Future.error(ResponseErrorMessage.backendError);
      }
    } else {
      return Future.error(ResponseErrorMessage.invalidedSecret);
    }
  }

  Future<UserModel> get({required String id}) async {
    if (_cryptoRevolutionMasterApp.isInitialize) {
      var headersList = {
        'Accept': '*/*',
      };
      var url = Uri.parse('${_cryptoRevolutionMasterApp.appUrl}/user/$id');

      var req = http.Request('GET', url);
      req.headers.addAll(headersList);

      var res = await req.send();
      final resBody = await res.stream.bytesToString();

      if (res.statusCode >= 200 && res.statusCode < 300) {
        Map<String, dynamic> resBodyJson = json.decode(resBody);
        if (resBodyJson.containsKey("data")) {
          return Future.value(
            UserModel(
              id: id, 
              email: resBodyJson["data"]["email"], 
              username: resBodyJson["data"]["username"], 
              password: resBodyJson["data"]["password"]
            )
          );
        } else {
          return Future.error(ResponseErrorMessage.userDoseNotExist);
        }
      }
      else {
        return Future.error(ResponseErrorMessage.backendError);
      }

    } else {
      return Future.error(ResponseErrorMessage.invalidedSecret);
    }
  }  

  Future<void> delete({required String id}) async {
    if (_cryptoRevolutionMasterApp.isInitialize) {
      var headersList = {
        'Accept': '*/*',
      };
      var url = Uri.parse('${_cryptoRevolutionMasterApp.appUrl}/user/$id');

      var req = http.Request('DELETE', url);
      req.headers.addAll(headersList);


      var res = await req.send();
      final resBody = await res.stream.bytesToString();

      if (res.statusCode >= 200 && res.statusCode < 300) {
        Map<String, dynamic> resBodyJson = json.decode(resBody);
        if (resBodyJson.containsKey("reason")) {
          return Future.error(ResponseErrorMessage.backendError);
        }
      }
      else {
        return Future.error(ResponseErrorMessage.backendError);
      }
    } else {
      return Future.error(ResponseErrorMessage.invalidedSecret);
    }
  }

  Future<void> buyCoin ({
    required String id,
    required String name,
    required num amount,
    required num currentPrice
  }) async {
    if (_cryptoRevolutionMasterApp.isInitialize) {
      var time = DateTime.now().toString();
      var headersList = {
        'Accept': '*/*',
        'Content-Type': 'application/json' 
      };

      var url = Uri.parse('${_cryptoRevolutionMasterApp.appUrl}/user/$id/portfolio/buy');

      var body = {
        "name": name,
        "amount": amount,
        "time": time,
        "current price": currentPrice
      };
      var req = http.Request('POST', url);
      req.headers.addAll(headersList);
      req.body = json.encode(body);

      var res = await req.send();
      final resBody = await res.stream.bytesToString();

      if (res.statusCode >= 200 && res.statusCode < 300) {
        Map<String, dynamic> resBodyJson = json.decode(resBody);
        if (resBodyJson.containsKey("reason")) {
          return Future.error(ResponseErrorMessage.userDoseNotExist);
        }
      }
      else {
        return Future.error(ResponseErrorMessage.backendError);
      }
    } else {
      return Future.error(ResponseErrorMessage.invalidedSecret);
    }
  }

  Future<void> sellCoin ({
    required String id,
    required String name,
    required num amount,
    required num currentPrice
  }) async {
    if (_cryptoRevolutionMasterApp.isInitialize) {
      var time = DateTime.now().toString();
      var headersList = {
        'Accept': '*/*',
        'Content-Type': 'application/json' 
      };

      var url = Uri.parse('${_cryptoRevolutionMasterApp.appUrl}/user/$id/portfolio/sell');

      var body = {
        "name": name,
        "amount": amount,
        "time": time,
        "current price": currentPrice
      };
      var req = http.Request('POST', url);
      req.headers.addAll(headersList);
      req.body = json.encode(body);

      var res = await req.send();
      final resBody = await res.stream.bytesToString();

      if (res.statusCode >= 200 && res.statusCode < 300) {
        Map<String, dynamic> resBodyJson = json.decode(resBody);
        if (resBodyJson.containsKey("reason")) {
          return Future.error(ResponseErrorMessage.userDoseNotExist);
        }
      }
      else {
        return Future.error(ResponseErrorMessage.backendError);
      }
    } else {
      return Future.error(ResponseErrorMessage.invalidedSecret);
    }
  }

  Future<UserModel> login ({
    required String emailOrUsername,
    required String password
  }) async {
    if (_cryptoRevolutionMasterApp.isInitialize) {
      var headersList = {
        'Accept': '*/*',
        'Content-Type': 'application/json' 
      };
      var url = Uri.parse('${_cryptoRevolutionMasterApp.appUrl}/user/login');

      var body = {
        "email_or_username": emailOrUsername,
        "password": password
      };
      var req = http.Request('POST', url);
      req.headers.addAll(headersList);
      req.body = json.encode(body);

      var res = await req.send();
      final resBody = await res.stream.bytesToString();

      if (res.statusCode >= 200 && res.statusCode < 300) {
        Map<String, dynamic> resBodyJson = json.decode(resBody);
        if (resBodyJson.containsKey("data")) {
          return Future.value(
            UserModel(
              id: resBodyJson["data"]["id"], 
              email: resBodyJson["data"]["email"], 
              username: resBodyJson["data"]["username"], 
              password: resBodyJson["data"]["password"]
            )
          );
        } else {
          if (resBodyJson["reason"] == "invalided details") {
            return Future.error(ResponseErrorMessage.invalidedDetails);
          } else {
            return Future.error(ResponseErrorMessage.backendError);
          }
        }
      }
      else {
        return Future.error(ResponseErrorMessage.backendError);
      }
    } else {
      return Future.error(ResponseErrorMessage.invalidedSecret);
    } 
  }

  Future<UserModel> signin({
    required String username,
    required String email,
    required String password
  }) async {
    if (_cryptoRevolutionMasterApp.isInitialize) {
      var headersList = {
        'Accept': '*/*',
        'Content-Type': 'application/json' 
      };
      var url = Uri.parse('${_cryptoRevolutionMasterApp.appUrl}/user/signin');

      var body = {
        "username": username,
        "password": password,
        "email": email
      };
      var req = http.Request('POST', url);
      req.headers.addAll(headersList);
      req.body = json.encode(body);

      var res = await req.send();
      final resBody = await res.stream.bytesToString();
      
      if (res.statusCode >= 200 && res.statusCode < 300) {
        Map<String, dynamic> resBodyJson = json.decode(resBody);
        if (resBodyJson.containsKey("data")) {
          return Future.value(
            UserModel(
              id: resBodyJson["data"]["id"], 
              email: resBodyJson["data"]["email"], 
              username: resBodyJson["data"]["username"], 
              password: resBodyJson["data"]["password"]
            )
          );
        } else {
          if (resBodyJson["reason"] == "email already exist") {
            return Future.error(ResponseErrorMessage.emailAlreadyExist);
          } else if (resBodyJson["reason"] == "username already exist") {
            return Future.error(ResponseErrorMessage.usernameAlreadyExist);
          } else {
            return Future.error(ResponseErrorMessage.backendError);
          }
        }
      }
      else {
        return Future.error(ResponseErrorMessage.backendError);
      }
    } else {
      return Future.error(ResponseErrorMessage.invalidedSecret);
    }
  }
}